#%% Load Packages & Define Functions
import os
import xarray as xr
import numpy as np
import matplotlib.pyplot as plt

def Get_Locations_with_Data(a_data):
    m_data = (~np.isnan(a_data)).any(dim='time')
    m_coords = np.empty([0,2])
    for i_lat in m_data.lat:
        for i_lon in m_data.lon:
            if m_data.sel(lat=i_lat,lon=i_lon):
                m_coords = np.r_[m_coords,np.array([i_lat,i_lon],ndmin=2)]          
    return m_coords

def Array_to_Matrix_at_Coords(a_data,m_coords):
    n_coords = m_coords.shape[0]
    if hasattr(a_data,'time'):
        n_times = len(a_data.time)
    else:
        n_times = 1
    m_data = np.empty([n_times,n_coords])
    m_data[:] = np.nan
    for i_coord in range(n_coords):
        m_data[:,i_coord] = np.array(a_data.to_array().sel(lat=m_coords[i_coord,0],lon=m_coords[i_coord,1]))
    return m_data

def Distance_Haversine(lat_1,lon_1,lat_2,lon_2):
    R = 6371 # radius of the earth in km
    lat_1 = np.radians(np.array(lat_1))
    lat_2 = np.radians(np.array(lat_2))
    lon_1 = np.radians(np.array(lon_1))
    lon_2 = np.radians(np.array(lon_2))
    delta_lat = lat_2 - lat_1
    delta_lon = lon_2 - lon_1
    a = (np.sin(delta_lat/2))*(np.sin(delta_lat/2)) + (np.cos(lat_1))*(np.cos(lat_2))*(np.sin(delta_lon/2))*(np.sin(delta_lon/2))
    # c = np.arctan2(np.sqrt(a),np.sqrt(1-a))
    c = np.arcsin(np.sqrt(a))
    d = 2*R*c
    return d

#%% Setup & Settings
s_region = 'New York City'
s_modeltype = 'Prediction Model (GEOS-CF + TROPOMI)'

lead_time = 1

d_regional_settings = {'New York City':'2A7L',\
                       'New Orleans':'2A7L',\
                       'Salt Lake City':'2A4L',\
                       'Las Vegas':'1A4L',\
                       'San Francisco':'2A7L'}

lib_lengthscale = {'NewYorkCity':35,'SaltLakeCity':20,'LasVegas':10,'NewOrleans':25,'SanFrancisco':30}
lib_residual_SD = {'NewYorkCity':8,'SaltLakeCity':5,'LasVegas':10,'NewOrleans':6,'SanFrancisco':6}
lib_timescale = {'NewYorkCity':5,'SaltLakeCity':4,'LasVegas':5,'NewOrleans':5,'SanFrancisco':6}

BASIC_PATH = os.path.join('C:\\','Data Archive')
OUTPUT_FIGURE_FOLDER = os.path.join(BASIC_PATH,'Generated Figures')

#%% Analysis & Plotting
s_regioncode = s_region.replace(' ','')
if (s_modeltype == 'Diurnal Model'):
    s_modelspec = s_modeltype
elif (s_modeltype == 'Persistance Model'):
    s_modelspec = s_modeltype
else:
    s_modelspec = s_modeltype + ' ' + d_regional_settings[s_region]

if ('Kriging' in s_modeltype):
    s_true_data_files = os.path.join(BASIC_PATH,'Generated Data',s_regioncode+' '+s_modelspec+' (truth).nc')
    s_prediction_data_files = os.path.join(BASIC_PATH,'Generated Data',s_regioncode+' '+s_modelspec+' (prediction).nc')
else:
    s_true_data_files = os.path.join(BASIC_PATH,'Generated Data',s_regioncode+' '+s_modelspec+' (truth).nc') 
    s_prediction_data_files = os.path.join(BASIC_PATH,'Generated Data',s_regioncode+' '+s_modelspec+' (prediction).nc')

OUTPUT_FIGURE_NAME = s_region + ' ' + s_modelspec + ' Residual Correlations.svg'
                
# Load Data
a_data_true = xr.open_dataset(s_true_data_files)
a_data_pred = xr.open_dataset(s_prediction_data_files)   

m_site_coords = Get_Locations_with_Data(a_data_true)
n_sites = m_site_coords.shape[0]

# Analysis of Residuals        
a_data_residual = a_data_pred.sel(leadtime=lead_time) - a_data_true
m_data_residual = Array_to_Matrix_at_Coords(a_data_residual,m_site_coords)

v_distances = np.empty([n_sites*n_sites])
v_spatial_correlations = np.empty([n_sites*n_sites])

v_distances[:] = np.nan
v_spatial_correlations[:] = np.nan

# Spatial Correlations
i_line = 0
for i_site_1 in range(n_sites):
    v_1 = m_data_residual[:,i_site_1]
    for i_site_2 in range(n_sites):
        v_2 = m_data_residual[:,i_site_2]
        
        id_num = ~np.isnan(np.c_[v_1,v_2]).any(axis=1)
        
        if (sum(id_num) > 24):
            v_distances[i_line] = Distance_Haversine(m_site_coords[i_site_1,0],m_site_coords[i_site_1,1],m_site_coords[i_site_2,0],m_site_coords[i_site_2,1])
            v_spatial_correlations[i_line] = np.corrcoef(v_1[id_num],v_2[id_num])[0,1]
            
        i_line += 1
    
v_distance_plotscale = np.arange(0,201)
v_fitted_spatial_correlations = np.exp(-v_distance_plotscale/lib_lengthscale[s_regioncode])


# Temporal Correlations
n_differences = (24*2)+1
v_differences = np.empty([n_sites,n_differences])
v_temporal_correlations = np.empty([n_sites,n_differences])

v_differences[:] = np.nan
v_temporal_correlations[:] = np.nan

for i_site in range(n_sites):
    for i_difference in range(n_differences):
        if i_difference == 0:
            v_1 = m_data_residual[:,i_site]
            v_2 = m_data_residual[:,i_site]
        else:
            v_1 = m_data_residual[:-i_difference,i_site]
            v_2 = m_data_residual[i_difference:,i_site]
        
        id_num = ~np.isnan(np.c_[v_1,v_2]).any(axis=1)
        if (sum(id_num) > 24):
            v_differences[i_site,i_difference] = i_difference
            v_temporal_correlations[i_site,i_difference] = np.corrcoef(v_1[id_num],v_2[id_num])[0,1]

v_difference_plotscale = np.arange(0,n_differences+1)
v_fitted_temporal_correlations = np.exp(-v_difference_plotscale/lib_timescale[s_regioncode])        



fig = plt.figure();   
fig.text(0.0, 0.97, 'a)', fontsize=12)
axes1 = fig.add_axes([0.15, 0.65, 0.83, 0.33]); # upper
fig.text(0.0, 0.47, 'b)', fontsize=12)
axes2 = fig.add_axes([0.15, 0.15, 0.83, 0.33]); # lower

# Plot Results - Spatial Correlations
axes1.scatter(x=v_distances,y=v_spatial_correlations)
axes1.plot(v_distance_plotscale,v_fitted_spatial_correlations,'r-')
axes1.set_ylim(-1,1)
axes1.set_xlim(0,100)
axes1.set_xlabel('distance [km]')
axes1.set_ylabel('residual correlation')

# Plot Results - Temporal Correlations
axes2.boxplot(v_temporal_correlations[~np.isnan(v_temporal_correlations).any(axis=1)])
axes2.plot(v_difference_plotscale+1,v_fitted_temporal_correlations,'r-')
axes2.set_xticks(range(1,n_differences+3))
axes2.set_xticklabels(range(0,n_differences+4),rotation=90, fontsize=8)
axes2.set_ylim(-1,1)
axes2.set_xlim(1,n_differences)
axes2.set_xlabel('time difference [hrs]')
axes2.set_ylabel('residual correlation')

fig.savefig(os.path.join(OUTPUT_FIGURE_FOLDER,OUTPUT_FIGURE_NAME),dpi=300)
fig.show()