#%% Load Packages & Define Functions
import os
import xarray as xr
import numpy as np
import matplotlib.pyplot as plt
import cartopy
from mpl_toolkits.axes_grid1.inset_locator import inset_axes

def Get_Locations_with_Data(a_data):
    m_data = (~np.isnan(a_data)).any(dim='time')
    m_coords = np.empty([0,2])
    for i_lat in m_data.lat:
        for i_lon in m_data.lon:
            if m_data.sel(lat=i_lat,lon=i_lon):
                m_coords = np.r_[m_coords,np.array([i_lat,i_lon],ndmin=2)]          
    return m_coords

def Array_to_Matrix_at_Coords(a_data,m_coords):
    n_coords = m_coords.shape[0]
    if hasattr(a_data,'time'):
        n_times = len(a_data.time)
    else:
        n_times = 1
    m_data = np.empty([n_times,n_coords])
    m_data[:] = np.nan
    for i_coord in range(n_coords):
        m_data[:,i_coord] = np.array(a_data.to_array().sel(lat=m_coords[i_coord,0],lon=m_coords[i_coord,1]))
    return m_data

def Regional_LatLon_Bounds(region_name):
    maxlat = np.inf
    maxlon = np.inf
    minlat = -np.inf
    minlon = -np.inf
    if region_name == 'CONUS':
        maxlat = 50 # dlat = 25
        maxlon = -65 # dlon = 60
        minlat = 25
        minlon = -125
    if region_name == 'SoCal':
        maxlat = 39 # dlat = 7
        maxlon = -115 # dlon = 8
        minlat = 32
        minlon = -123
    if region_name == 'UtCoWy':
        maxlat = 46 # dlat = 8
        maxlon = -104 # dlon = 9
        minlat = 38
        minlon = -113
    if region_name == 'NorthEast':
        maxlat = 43 # dlat = 5
        maxlon = -70 # dlon = 11
        minlat = 38
        minlon = -81
    if region_name == 'GulfCoast':
        maxlat = 35 # dlat = 7
        maxlon = -88 # dlon = 11
        minlat = 28
        minlon = -99
    if region_name == 'New York City':
        maxlat = 42 # dlat = 2
        maxlon = -73 # dlon = 2
        minlat = 40
        minlon = -75
    if region_name == 'Boston':
        maxlat = 43 # dlat = 2
        maxlon = -70 # dlon = 2
        minlat = 41
        minlon = -72
    if region_name == 'New Orleans':
        maxlat = 31 # dlat = 2
        maxlon = -90 # dlon = 2
        minlat = 29
        minlon = -92
    if region_name == 'Houston':
        maxlat = 31 # dlat = 2
        maxlon = -97 # dlon = 2
        minlat = 29
        minlon = -99
    if region_name == 'Salt Lake City':
        maxlat = 42 # dlat = 2
        maxlon = -111 # dlon = 2
        minlat = 40
        minlon = -113
    if region_name == 'Denver':
        maxlat = 41 # dlat = 2
        maxlon = -104 # dlon = 2
        minlat = 39
        minlon = -106
    if region_name == 'San Francisco':
        maxlat = 39 # dlat = 2
        maxlon = -121 # dlon = 2
        minlat = 37
        minlon = -123
    if region_name == 'Los Angeles':
        maxlat = 35 # dlat = 2
        maxlon = -116 # dlon = 4
        minlat = 33
        minlon = -120
    if region_name == 'Las Vegas':
        maxlat = 37 # dlat = 2
        maxlon = -114 # dlon = 2
        minlat = 35
        minlon = -116
    return maxlat,maxlon,minlat,minlon

#%% Basic Settings
        
figure_format = '.svg'

BASIC_FOLDER = os.path.join('C:\\','Data Archive')

OUTPUT_FIGURE_FOLDER = os.path.join(BASIC_FOLDER,'Generated Figures')

l_regions = ['Las Vegas','New Orleans','New York City','Salt Lake City','San Francisco']

SUBSET = 4

if SUBSET == 1:
    l_modeltypes = ['Persistance Model','Prediction Model (GEOS-CF + TROPOMI)']    
    l_lead_time = [0,0]
    d_metricchangerange = {'RMSE':[-20,20],\
                           'MAE':[-20,20],\
                           'bias':[-5,5],\
                           'AB':[-10,10],\
                           'r':[-1,1]}
    
if SUBSET == 2:
    l_modeltypes = ['Persistance Model','Prediction Model (GEOS-CF + TROPOMI)']    
    l_lead_time = [6,6]
    d_metricchangerange = {'RMSE':[-20,20],\
                           'MAE':[-20,20],\
                           'bias':[-5,5],\
                           'AB':[-10,10],\
                           'r':[-1,1]}
    
if SUBSET == 3:
    l_modeltypes = ['Prediction Model (GEOS-CF)','Prediction Model (GEOS-CF + TROPOMI)']    
    l_lead_time = [6,6]
    d_metricchangerange = {'RMSE':[-5,5],\
                           'MAE':[-5,5],\
                           'bias':[-5,5],\
                           'AB':[-5,5],\
                           'r':[-0.5,0.5]}
    
if SUBSET == 4:
    l_modeltypes = ['Prediction Model (GEOS-CF + TROPOMI)','Prediction & Kriging Model (GEOS-CF + TROPOMI)']    
    l_lead_time = [1,1]
    d_metricchangerange = {'RMSE':[-2,2],\
                           'MAE':[-2,2],\
                           'bias':[-5,5],\
                           'AB':[-2,2],\
                           'r':[-0.5,0.5]}

d_modeltype_labels = {'Persistance Model':'Persistence Baseline',\
                  'Diurnal Model':'Climatology Baseline',\
                  'Prediction Model (GEOS-CF)':'GEOS-CF',\
                  'Prediction Model (GEOS-CF + TROPOMI)':'GEOS-CF + TROPOMI',\
                  'Prediction & Kriging Model (GEOS-CF + TROPOMI)':'GEOS-CF + TROPOMI & Kriging',\
                  'Prediction Model (GEOS-CF + VPN46A1)':'GEOS-CF + VIIRS',\
                  'Prediction Model (GEOS-CF + TROPOMI + VPN46A1)':'GEOS-CF + TROPOMI + VIIRS',\
                  'Prediction Model (GEOS-CF + TROPOMI + MET)':'GEOS-CF + TROPOMI + MET'}


# l_lead_time = [0,1,3,6,9,12,15,18,21,24]
# l_lead_time_labels = ['0 hr',' 1 hr',' 3 hr',' 6 hr',' 9 hr','12 hr','15 hr','18 hr','21 hr','24 hr']

d_lead_time_labels = {0:' 0 hr',\
                      1:' 1 hr',\
                      3:' 3 hr',\
                      6:' 6 hr',\
                      9:' 9 hr',\
                      12:'12 hr',\
                      15:'15 hr',\
                      18:'18 hr',\
                      21:'21 hr',\
                      24:'24 hr'}

d_regional_colors = {'New York City':'blue',\
                       'New Orleans':'cyan',\
                       'Salt Lake City':'orange',\
                       'Las Vegas':'black',\
                       'San Francisco':'red'}
    
d_regional_settings = {'New York City':'2A7L',\
                       'New Orleans':'2A7L',\
                       'Salt Lake City':'2A4L',\
                       'Las Vegas':'1A4L',\
                       'San Francisco':'2A7L'}

l_metricsets = ['r','AB','bias','RMSE','MAE']
# l_metricsets = ['r']    




d_metricvaluerange = {'RMSE':[0,20],\
                       'MAE':[0,20],\
                       'bias':[-5,5],\
                       'AB':[0,10],\
                       'r':[0,1]}

        
d_metriclabels = {'RMSE':'Change in RMSE [ppb]',\
                  'MAE':'Change in MAE [ppb]',\
                  'bias':'Change in bias [ppb]',\
                  'AB':'Change in AB [ppb]',\
                  'r':'Change in correlation'}

d_metric_colormaps = {'RMSE':plt.cm.get_cmap('bwr'),\
                      'MAE':plt.cm.get_cmap('bwr'),\
                      'bias':plt.cm.get_cmap('Spectral').reversed(),\
                      'AB':plt.cm.get_cmap('bwr'),\
                      'r':plt.cm.get_cmap('bwr').reversed()}
        
#%% Compute Metrics
n_max_sites = 30

# a_metrics = np.zeros([n_max_sites,len(l_regions),len(l_lead_time),len(l_modeltypes),len(l_metricsets)])
a_metrics = np.zeros([n_max_sites,len(l_regions),len(l_modeltypes),len(l_metricsets)])
a_metrics[:] = np.nan

a_site_coords = np.zeros([2,n_max_sites,len(l_regions)])
a_site_coords[:] = np.nan

for i_region in range(len(l_regions)):
    s_region = l_regions[i_region]
    s_regioncode = s_region.replace(' ','')
    s_spec = d_regional_settings[s_region]
    l_true_data_files = []
    l_prediction_data_files = []
    for i_model in range(len(l_modeltypes)):
        s_modeltype = l_modeltypes[i_model]
        basepath = os.path.join(BASIC_FOLDER,'Generated Data')
        if (s_modeltype == 'Diurnal Model'):
            l_true_data_files += [os.path.join(basepath,s_regioncode+' '+s_modeltype +' (truth).nc')]
            l_prediction_data_files += [os.path.join(basepath,s_regioncode+' '+s_modeltype +' (prediction).nc')]
        elif (s_modeltype == 'Persistance Model'):
            l_true_data_files += [os.path.join(basepath,s_regioncode+' '+s_modeltype +' (truth).nc')]
            l_prediction_data_files += [os.path.join(basepath,s_regioncode+' '+s_modeltype +' (prediction).nc')]
        else:
            l_true_data_files += [os.path.join(basepath,s_regioncode+' '+s_modeltype +' '+s_spec+' (truth).nc')]
            l_prediction_data_files += [os.path.join(basepath,s_regioncode+' '+s_modeltype +' '+s_spec+' (prediction).nc')]

    for i_dataset in range(len(l_true_data_files)):
        # Load Data
        a_data_true = xr.open_dataset(l_true_data_files[i_dataset])
        a_data_pred = xr.open_dataset(l_prediction_data_files[i_dataset])
        
        # Compute Metrics
        m_site_coords = Get_Locations_with_Data(a_data_true)
        n_sites = m_site_coords.shape[0]
        m_data_true = Array_to_Matrix_at_Coords(a_data_true,m_site_coords)
        
        #for i_leadtime in range(len(l_lead_time)):
            #lead_time = l_lead_time[i_leadtime]
        lead_time = l_lead_time[i_dataset]
        
        v_results_r = np.empty([n_sites,1])
        v_results_MAE = np.empty([n_sites,1])
        v_results_RMSE = np.empty([n_sites,1])
        v_results_bias = np.empty([n_sites,1])
        v_results_AB = np.empty([n_sites,1])
        v_results_MAPE = np.empty([n_sites,1])
        v_results_RMSPE = np.empty([n_sites,1])
        v_results_Pbias = np.empty([n_sites,1])
        
        v_results_r[:] = np.nan
        v_results_MAE[:] = np.nan
        v_results_MAPE[:] = np.nan
        v_results_RMSE[:] = np.nan
        v_results_RMSPE[:] = np.nan
        v_results_bias[:] = np.nan
        v_results_AB[:] = np.nan
        v_results_Pbias[:] = np.nan
        
        m_data_pred = Array_to_Matrix_at_Coords(a_data_pred.sel(leadtime=lead_time),m_site_coords)
    
        for site_index in range(n_sites):
            v_y = m_data_true[:,site_index]
            v_y_hat = m_data_pred[:,site_index]
           
            v_error = v_y - v_y_hat
            v_Perror = np.abs(v_error/v_y)
                    
            id_num = ~np.isnan(np.c_[v_y,v_y_hat]).any(axis=1)
        
            if (sum(id_num) > 24):
                v_results_r[site_index] = np.corrcoef(v_y[id_num],v_y_hat[id_num])[0,1]
                v_results_MAE[site_index] = np.nanmean(np.abs(v_error))
                v_results_RMSE[site_index] = np.sqrt(np.nanmean(v_error**2))
                v_results_bias[site_index] = np.nanmean(v_error)
                v_results_AB[site_index] = np.abs(np.nanmean(v_error))
                v_results_MAPE[site_index] = np.nanmean(np.abs(v_Perror))
                v_results_RMSPE[site_index] = np.sqrt(np.nanmean(v_Perror**2))
                v_results_Pbias[site_index] = np.nanmean(v_error)/np.nanmean(v_y)
                
        for i_metric in range(len(l_metricsets)):
            metric = l_metricsets[i_metric]
            # Identify PoI
            if (metric == 'r'):
                v_metric = v_results_r
            elif (metric == 'RMSE'):
                v_metric = v_results_RMSE
            elif (metric == 'MAE'):
                v_metric = v_results_MAE
            elif (metric == 'bias'):
                v_metric = v_results_bias
            elif (metric == 'AB'):
                v_metric = v_results_AB
            elif (metric == 'MAPE'):
                v_metric = v_results_MAPE
            elif (metric == 'RMSPE'):
                v_metric = v_results_RMSPE
            elif (metric == 'Pbias'):
                v_metric = v_results_Pbias
                
            valid_site_index = 0
            for site_index in range(n_sites):
                if ~np.isnan(v_metric[site_index]):
                    a_site_coords[0,valid_site_index,i_region] = m_site_coords[site_index,0]
                    a_site_coords[1,valid_site_index,i_region] = m_site_coords[site_index,1]
                    # a_metrics[valid_site_index,i_region,i_leadtime,i_dataset,i_metric] = v_metric[site_index]
                    a_metrics[valid_site_index,i_region,i_dataset,i_metric] = v_metric[site_index]
                    valid_site_index += 1
                    
#%% Generate Plots

a_axes = [[0.03,0.54,0.3,0.38],\
          [0.36,0.54,0.3,0.38],\
          [0.69,0.54,0.3,0.38],\
          [0.03,0.04,0.3,0.38],\
          [0.36,0.04,0.3,0.38]]

v_subplot_labels = ['a)','b)','c)','d)','e)']    
a_subplot_labels = [[  0,0.95],\
                    [0.32,0.95],\
                    [0.64,0.95],\
                    [  0,0.45],\
                    [0.32,0.45]]
    
for i_metric in range(len(l_metricsets)):
    # Create Figure
    fig = plt.figure(figsize=(8,5));
    
    for i_region in range(len(l_regions)):
        n_max_lat,n_max_lon,n_min_lat,n_min_lon = Regional_LatLon_Bounds(l_regions[i_region])
        
        # Create Subfigure
        fig.text(a_subplot_labels[i_region][0],a_subplot_labels[i_region][1],v_subplot_labels[i_region],fontsize=12)
        axis = fig.add_axes(a_axes[i_region],projection=cartopy.crs.PlateCarree())
        # axis = fig.add_subplot(2,3,i_region+1,projection=cartopy.crs.PlateCarree(),constrained_layout=True);
        
        
        # Compute differences
        n_sites = np.sum(~np.isnan(a_site_coords[0,:,i_region]))
        m_coords = a_site_coords[:,0:n_sites,i_region]
        v_site_values_0 = np.zeros([n_sites])
        v_site_values_1 = np.zeros([n_sites])
        v_site_values_0[:] = np.nan
        v_site_values_1[:] = np.nan
        
        for i_site in range(n_sites):
            v_site_values_0[i_site] = a_metrics[i_site,i_region,0,i_metric]
            v_site_values_1[i_site] = a_metrics[i_site,i_region,1,i_metric]
        v_site_differences = v_site_values_1 - v_site_values_0
        
        # Plot Differences
        axis.coastlines();
        axis.add_feature(cartopy.feature.LAKES, edgecolor='black', facecolor='none', linestyle='-');
        axis.set_extent([np.max([n_min_lon,-180])-0.001, np.min([n_max_lon,180])+0.001, np.max([n_min_lat,-90])-0.001, np.min([n_max_lat,90])+0.001])
        gridlines = axis.gridlines(crs=cartopy.crs.PlateCarree(), xlocs=range(np.int_(np.floor(np.max([n_min_lon,-180]))),1+np.int_(np.ceil(np.min([n_max_lon,180])))), ylocs=range(np.int_(np.floor(np.max([n_min_lat,-90]))),1+np.int_(np.ceil(np.min([n_max_lat,90])))), draw_labels=True, linewidth=1, color='black', alpha=0.5, linestyle='--')
        gridlines.top_labels = False
        gridlines.right_labels = False
        # my_cmap = plt.cm.get_cmap('bwr').reversed()
        my_cmap = d_metric_colormaps[l_metricsets[i_metric]]
        scatterplot = plt.scatter(x=m_coords[1,:],y=m_coords[0,:],c=v_site_differences,vmin=d_metricchangerange[l_metricsets[i_metric]][0],vmax=d_metricchangerange[l_metricsets[i_metric]][1],cmap=my_cmap)
        plt.title(l_regions[i_region])

        # Legend
        if i_region == (len(l_regions) - 1):
            # Plot Legend
            axins = inset_axes(axis,width="5%",height="100%",loc='center left',bbox_to_anchor=(1.1, 0, 1, 1),bbox_transform=axis.transAxes,borderpad=0)
            plt.colorbar(scatterplot, cax=axins).set_label(d_metriclabels[l_metricsets[i_metric]])
            
    # Save Plot  
    OUTPUT_FIGURE_NAME = 'Location Comparison - ' + l_modeltypes[1] + ' ' + d_lead_time_labels[l_lead_time[1]] + ' v. ' + l_modeltypes[0] + ' ' + d_lead_time_labels[l_lead_time[0]] + ' - ' + l_metricsets[i_metric] + figure_format
    fig.savefig(os.path.join(OUTPUT_FIGURE_FOLDER,OUTPUT_FIGURE_NAME),dpi=300)
    fig.show()